@@ -54,7 +54,7 @@ mod circuit {
         }
 
         dep_obj! {
-            pub dyn fn legs(self as this, circuit: Circuit) -> ChipLegs {
+            pub fn legs(self as this, circuit: Circuit) -> dyn ChipLegs {
                 if mut { &mut circuit.arena[this.0].legs } else { &circuit.arena[this.0].legs }
             }
         }
@@ -90,7 +90,7 @@ mod or_chip {
 
     dep_type! {
         #[derive(Debug)]
-        pub struct OrLegs become legs in Chip {
+        pub struct OrLegs in Chip {
             in_1: bool = false,
             in_2: bool = false,
             out: bool = false,
@@ -131,7 +131,7 @@ mod not_chip {
 
     dep_type! {
         #[derive(Debug)]
-        pub struct NotLegs become legs in Chip {
+        pub struct NotLegs in Chip {
             in_: bool = false,
             out: bool = true,
         }
