@@ -15,6 +15,106 @@
 extern crate core;
 extern crate alloc;
 
+#[cfg(docsrs)]
+pub mod example {
+    //! The [`dep_type`] and [`dep_obj`] macro expansion example.
+    //!
+    //! ```ignore
+    //! dep_type! {
+    //!     #[derive(Debug)]
+    //!     pub struct MyDepType in MyDepTypeId {
+    //!         prop_1: bool = false,
+    //!         prop_2: i32 = 10,
+    //!     }
+    //! }
+    //!
+    //! macro_attr! {
+    //!     #[derive(Component!, Debug)]
+    //!     struct MyDepTypePrivateData {
+    //!         dep_data: MyDepType,
+    //!     }
+    //! }
+    //!
+    //! macro_attr! {
+    //!     #[derive(ComponentId!, Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
+    //!     pub struct MyDepTypeId(Id<MyDepTypePrivateData>);
+    //! }
+    //!
+    //! macro_attr! {
+    //!     #[derive(Context!, Debug)]
+    //!     pub struct MyApp {
+    //!         my_dep_types: Arena<MyDepTypePrivateData>,
+    //!     }
+    //! }
+    //!
+    //! impl MyDepTypeId {
+    //!     pub fn new(app: &mut MyApp) -> MyDepTypeId {
+    //!         app.my_dep_types.insert(|id| (MyDepTypePrivateData {
+    //!             dep_data: MyDepType::new_priv()
+    //!         }, MyDepTypeId(id)))
+    //!     }
+    //!
+    //!     dep_obj! {
+    //!         pub fn obj(self as this, app: MyApp) -> MyDepType {
+    //!             if mut {
+    //!                 &mut app.my_dep_types[this.0].dep_data
+    //!             } else {
+    //!                 &app.my_dep_types[this.0].dep_data
+    //!             }
+    //!         }
+    //!     }
+    //! }
+
+    use crate::{dep_obj, dep_type};
+    use components_arena::{Arena, Component, ComponentId, Id};
+    use dyn_context::Context;
+
+    dep_type! {
+        #[derive(Debug)]
+        pub struct MyDepType in MyDepTypeId {
+            prop_1: bool = false,
+            prop_2: i32 = 10,
+        }
+    }
+
+    #[derive(Debug)]
+    struct MyDepTypePrivateData {
+        dep_data: MyDepType,
+    }
+
+    Component!(() struct MyDepTypePrivateData { .. });
+
+    #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
+    pub struct MyDepTypeId(Id<MyDepTypePrivateData>);
+
+    ComponentId!(() pub struct MyDepTypeId(Id<MyDepTypePrivateData>););
+
+    #[derive(Debug)]
+    pub struct MyApp {
+        my_dep_types: Arena<MyDepTypePrivateData>,
+    }
+
+    Context!(() pub struct MyApp { .. });
+
+    impl MyDepTypeId {
+        pub fn new(app: &mut MyApp) -> MyDepTypeId {
+            app.my_dep_types.insert(|id| (MyDepTypePrivateData {
+                dep_data: MyDepType::new_priv()
+            }, MyDepTypeId(id)))
+        }
+
+        dep_obj! {
+            pub fn obj(self as this, app: MyApp) -> MyDepType {
+                if mut {
+                    &mut app.my_dep_types[this.0].dep_data
+                } else {
+                    &app.my_dep_types[this.0].dep_data
+                }
+            }
+        }
+    }
+}
+
 #[doc(hidden)]
 pub use core::compile_error as std_compile_error;
 #[doc(hidden)]
@@ -50,6 +150,8 @@ use dyn_context::{Context, ContextExt};
 use educe::Educe;
 use phantom_type::PhantomType;
 
+/// A type should satisfy this trait to be a dependency property type
+/// or a dependency vector item type.
 pub trait DepPropType: Clone + Debug + Send + Sync + 'static { }
 
 impl<PropType: Clone + Debug + Send + Sync + 'static> DepPropType for PropType { }
@@ -75,9 +177,9 @@ impl<OwnerId: ComponentId, PropType: DepPropType> DepPropEntry<OwnerId, PropType
     }
 }
 
-#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+#[derive(Debug, Clone)]
 pub enum DepVecChange<ItemType: DepPropType> {
-    Reset,
+    Reset(Vec<ItemType>),
     Inserted(Range<usize>),
     Removed(usize, Vec<ItemType>),
     Swapped(Range<usize>, Range<usize>),
@@ -113,6 +215,77 @@ impl<OwnerId: ComponentId, ArgsType> DepEventEntry<OwnerId, ArgsType> {
     }
 }
 
+/// A dependency type.
+/// Use the [`dep_type`] or the [`dep_type_with_builder`] macro
+/// to create a type implementing this trait.
+///
+/// # Examples
+///
+/// ```rust
+/// # #![feature(const_maybe_uninit_as_ptr)]
+/// # #![feature(const_ptr_offset_from)]
+/// # #![feature(const_raw_ptr_deref)]
+/// use components_arena::{Arena, Component, ComponentClassToken, ComponentId, Id};
+/// use dep_obj::{dep_obj, dep_type};
+/// use dyn_context::Context;
+/// use macro_attr_2018::macro_attr;
+///
+/// dep_type! {
+///     #[derive(Debug)]
+///     pub struct MyDepType in MyDepTypeId {
+///         prop_1: bool = false,
+///         prop_2: i32 = 10,
+///     }
+/// }
+///
+/// macro_attr! {
+///     #[derive(Component!, Debug)]
+///     struct MyDepTypePrivateData {
+///         dep_data: MyDepType,
+///     }
+/// }
+///
+/// macro_attr! {
+///     #[derive(ComponentId!, Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
+///     pub struct MyDepTypeId(Id<MyDepTypePrivateData>);
+/// }
+///
+/// macro_attr! {
+///     #[derive(Context!, Debug)]
+///     pub struct MyApp {
+///         my_dep_types: Arena<MyDepTypePrivateData>,
+///     }
+/// }
+///
+/// impl MyDepTypeId {
+///     pub fn new(app: &mut MyApp) -> MyDepTypeId {
+///         app.my_dep_types.insert(|id| (MyDepTypePrivateData {
+///             dep_data: MyDepType::new_priv()
+///         }, MyDepTypeId(id)))
+///     }
+///
+///     dep_obj! {
+///         pub fn obj(self as this, app: MyApp) -> MyDepType {
+///             if mut {
+///                 &mut app.my_dep_types[this.0].dep_data
+///             } else {
+///                 &app.my_dep_types[this.0].dep_data
+///             }
+///         }
+///     }
+/// }
+///
+/// fn main() {
+///     let mut my_dep_types_token = ComponentClassToken::new().unwrap();
+///     let mut app = MyApp {
+///         my_dep_types: Arena::new(&mut my_dep_types_token),
+///     };
+///     let id = MyDepTypeId::new(&mut app);
+///     assert_eq!(id.obj_ref(&app).get(MyDepType::PROP_2), &10);
+///     id.obj_mut(&mut app).set_distinct(MyDepType::PROP_2, 5);
+///     assert_eq!(id.obj_ref(&app).get(MyDepType::PROP_2), &5);
+/// }
+/// ```
 pub trait DepType: Sized {
     type Id: ComponentId;
 
@@ -120,6 +293,7 @@ pub trait DepType: Sized {
     fn style__(&mut self) -> &mut Option<Style<Self>>;
 }
 
+/// A dependency property.
 #[derive(Educe)]
 #[educe(Debug, Clone, Copy)]
 pub struct DepProp<Owner: DepType, PropType: DepPropType> {
@@ -128,6 +302,8 @@ pub struct DepProp<Owner: DepType, PropType: DepPropType> {
 }
 
 impl<Owner: DepType, PropType: DepPropType> DepProp<Owner, PropType> {
+    /// Creates dependency property. The only safe way to call this function is through
+    /// the [`dep_type`] or the [`dep_type_with_builder`] macro using.
     pub const unsafe fn new(offset: usize) -> Self {
         DepProp { offset, _phantom: PhantomType::new() }
     }
@@ -258,6 +434,9 @@ impl<Owner: DepType, PropType: DepPropType> AnySetter<Owner> for Setter<Owner, P
     }
 }
 
+/// A dictionary mapping a subset of target type properties to the values.
+/// Every dependency object can have an applied style at every moment.
+/// To switch an applied style, use the [`DepObjMut::apply_style`] function.
 #[derive(Educe)]
 #[educe(Debug, Clone, Default)]
 pub struct Style<Owner: DepType> {
@@ -549,8 +728,8 @@ impl<'a, Owner: DepType, Arena: 'static> DepObjMut<'a, Owner, Arena> {
         let arena: &mut Arena = self.context.get_mut();
         let obj = (self.get_obj_mut)(arena, self.id);
         let entry_mut = vec.entry_mut(obj);
-        entry_mut.items.clear();
-        let change = DepVecChange::Reset;
+        let old_items = replace(&mut entry_mut.items, Vec::new());
+        let change = DepVecChange::Reset(old_items);
         for on_changed in entry_mut.on_changed.clone() {
             on_changed(self.context, self.id, &change);
         }
@@ -613,12 +792,22 @@ impl<'a, Owner: DepType, Arena: 'static> DepObjMut<'a, Owner, Arena> {
 }
 
 #[macro_export]
-macro_rules! dep_type {
+macro_rules! dep_type_with_builder {
+    (
+        $($token:tt)*
+    ) => {
+        $crate::dep_type_with_builder_impl! { $($token)* }
+    };
+}
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! dep_type_with_builder_impl {
     (
         type BuilderCore $($token:tt)*
     ) => {
         $crate::generics_parse! {
-            $crate::dep_type {
+            $crate::dep_type_with_builder_impl {
                 @type BuilderCore
             }
         }
@@ -632,7 +821,7 @@ macro_rules! dep_type {
         $(#[$attr:meta])* $vis:vis struct $name:ident $($body:tt)*
     ) => {
         $crate::generics_parse! {
-            $crate::dep_type {
+            $crate::dep_type_with_builder_impl {
                 @struct
                 [[$BuilderCore] [$($bc_g)*] [$($bc_r)*] [$($bc_w)*]]
                 [$([$attr])*] [$vis] [$name]
@@ -660,14 +849,14 @@ macro_rules! dep_type {
     ) => {
         $crate::std_compile_error!("\
             invalid dep type builder core definition; allowed form is \
-            'type BuilderCore $(<$generics> $($where_clause)?)? = $builder_core_type;
+            'type BuilderCore $(<$generics> $($where_clause)?)? = $builder_core_type;\
         ");
     };
     (
         $(#[$attr:meta])* $vis:vis struct $name:ident $($body:tt)*
     ) => {
         $crate::generics_parse! {
-            $crate::dep_type {
+            $crate::dep_type_with_builder_impl {
                 @struct
                 []
                 [$([$attr])*] [$vis] [$name]
@@ -677,7 +866,7 @@ macro_rules! dep_type {
     };
     (
         @struct
-        [$([$BuilderCore:ty] [$($bc_g:tt)*] [$($bc_r:tt)*] [$($bc_w:tt)*])?]
+        [[$BuilderCore:ty] [$($bc_g:tt)*] [$($bc_r:tt)*] [$($bc_w:tt)*]]
         [$([$attr:meta])*] [$vis:vis] [$name:ident]
         [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
         become $obj:ident in $Id:ty
@@ -685,11 +874,11 @@ macro_rules! dep_type {
             $($($field:ident $delim:tt $($field_ty:ty $(= $field_val:expr)?)?),+ $(,)?)?
         }
     ) => {
-        $crate::dep_type! {
+        $crate::dep_type_with_builder_impl! {
             @concat_generics
             [$([$attr])*] [$vis] [$name] [$obj] [$Id]
             [$($g)*] [$($r)*] [$($w)*]
-            [$([$BuilderCore] [$($bc_g)*] [$($bc_r)*] [$($bc_w)*])?]
+            [[$BuilderCore] [$($bc_g)*] [$($bc_r)*] [$($bc_w)*]]
             [$($([$field $delim $($field_ty $(= $field_val)?)?])+)?]
         }
     };
@@ -719,7 +908,7 @@ macro_rules! dep_type {
         type BuilderCore $($token:tt)*
     ) => {
         $crate::generics_parse! {
-            $crate::dep_type {
+            $crate::dep_type_with_builder_impl {
                 @type BuilderCore after
                 [$([$attr])*] [$vis] [$name] [$obj] [$Id]
                 [$($g)*] [$($r)*] [$($w)*]
@@ -728,6 +917,22 @@ macro_rules! dep_type {
             $($token)*
         }
     };
+    (
+        @struct
+        []
+        [$([$attr:meta])*] [$vis:vis] [$name:ident]
+        [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
+        become $obj:ident in $Id:ty
+        {
+            $($($field:ident $delim:tt $($field_ty:ty $(= $field_val:expr)?)?),+ $(,)?)?
+        }
+    ) => {
+        $crate::std_compile_error!("\
+            missing dep type builder core definition; add the definition in the following form \
+            before or after dep type definition: \
+            'type BuilderCore $(<$generics> $($where_clause)?)? = $builder_core_type;\
+        ");
+    };
     (
         @struct
         []
@@ -771,7 +976,7 @@ macro_rules! dep_type {
         [$($bc_g:tt)*] [$($bc_r:tt)*] [$($bc_w:tt)*]
         = $BuilderCore:ty;
     ) => {
-        $crate::dep_type! {
+        $crate::dep_type_with_builder_impl! {
             @concat_generics
             [$([$attr])*] [$vis] [$name] [$obj] [$Id]
             [$($g)*] [$($r)*] [$($w)*]
@@ -812,7 +1017,7 @@ macro_rules! dep_type {
         [$([$field:ident $delim:tt $($field_ty:ty $(= $field_val:expr)?)?])*]
     ) => {
         $crate::generics_concat! {
-            $crate::dep_type {
+            $crate::dep_type_with_builder_impl {
                 @concat_generics_done
                 [$BuilderCore]
                 [$([$attr])*] [$vis] [$name] [$obj] [$Id]
@@ -831,7 +1036,7 @@ macro_rules! dep_type {
         [$([$field:ident $delim:tt $($field_ty:ty $(= $field_val:expr)?)?])*]
         [$($bc_g:tt)*] [$($bc_r:tt)*] [$($bc_w:tt)*]
     ) => {
-        $crate::dep_type! {
+        $crate::dep_type_impl_raw! {
             @unroll_fields
             [$([$attr])*] [$vis] [$name] [$obj] [$Id]
             [$($g)*] [$($r)*] [$($w)*]
@@ -841,21 +1046,114 @@ macro_rules! dep_type {
         }
     };
     (
-        @concat_generics
-        [$([$attr:meta])*] [$vis:vis] [$name:ident] [$obj:ident] [$Id:ty]
-        [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
-        []
-        [$([$field:ident $delim:tt $($field_ty:ty $(= $field_val:expr)?)?])*]
+        $($token:tt)*
     ) => {
-        $crate::dep_type! {
+        $crate::std_compile_error!("\
+            invalid dep type definition, allowed form is\n\
+            \n\
+            $(#[$attr])* $vis struct $name $(<$generics> $(where $where_clause)?)? become $obj in $Id {\n\
+                $field_1_name $(: $field_1_type = $field_1_value | [$field_1_type] | yield $field_1_type),\n\
+                $field_2_name $(: $field_2_type = $field_2_value | [$field_2_type] | yield $field_2_type),\n\
+                ...\n\
+            }\n\
+            \n\
+        ");
+    };
+}
+
+#[macro_export]
+macro_rules! dep_type {
+    (
+        $($token:tt)*
+    ) => {
+        $crate::dep_type_impl! { $($token)* }
+    };
+}
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! dep_type_impl {
+    (
+        $(#[$attr:meta])* $vis:vis struct $name:ident $($body:tt)*
+    ) => {
+        $crate::generics_parse! {
+            $crate::dep_type_impl {
+                @struct
+                []
+                [$([$attr])*] [$vis] [$name]
+            }
+            $($body)*
+        }
+    };
+    (
+        @struct
+        []
+        [$([$attr:meta])*] [$vis:vis] [$name:ident]
+        [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
+        in $Id:ty
+        {
+            $($($field:ident $delim:tt $($field_ty:ty $(= $field_val:expr)?)?),+ $(,)?)?
+        }
+    ) => {
+        $crate::dep_type_impl_raw! {
             @unroll_fields
-            [$([$attr])*] [$vis] [$name] [$obj] [$Id]
+            [$([$attr])*] [$vis] [$name] [obj] [$Id]
             [$($g)*] [$($r)*] [$($w)*]
             [] [] [] []
             []
-            [$([$field $delim $($field_ty $(= $field_val)?)?])*]
+            [$($([$field $delim $($field_ty $(= $field_val)?)?])+)?]
         }
     };
+    (
+        @struct
+        []
+        [$([$attr:meta])*] [$vis:vis] [$name:ident]
+        [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
+        in $Id:ty
+        {
+            $($($field:ident $delim:tt $($field_ty:ty $(= $field_val:expr)?)?),+ $(,)?)?
+        }
+        $($token:tt)+
+    ) => {
+        $crate::std_compile_error!("unexpected extra tokens after dep type definition body");
+    };
+    (
+        @struct
+        []
+        [$([$attr:meta])*] [$vis:vis] [$name:ident]
+        [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
+        $($token:tt)*
+    ) => {
+        $crate::std_compile_error!("\
+            invalid dep type definition, allowed form is\n\
+            \n\
+            $(#[$attr])* $vis struct $name $(<$generics> $(where $where_clause)?)? in $Id {\n\
+                $field_1_name $(: $field_1_type = $field_1_value | [$field_1_type] | yield $field_1_type),\n\
+                $field_2_name $(: $field_2_type = $field_2_value | [$field_2_type] | yield $field_2_type),\n\
+                ...\n\
+            }\n\
+            \n\
+        ");
+    };
+    (
+        $($token:tt)*
+    ) => {
+        $crate::std_compile_error!("\
+            invalid dep type definition, allowed form is\n\
+            \n\
+            $(#[$attr])* $vis struct $name $(<$generics> $(where $where_clause)?)? in $Id {\n\
+                $field_1_name $(: $field_1_type = $field_1_value | [$field_1_type] | yield $field_1_type),\n\
+                $field_2_name $(: $field_2_type = $field_2_value | [$field_2_type] | yield $field_2_type),\n\
+                ...\n\
+            }\n\
+            \n\
+        ");
+    };
+}
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! dep_type_impl_raw {
     (
         @unroll_fields
         [$([$attr:meta])*] [$vis:vis] [$name:ident] [$obj:ident] [$Id:ty]
@@ -870,7 +1168,7 @@ macro_rules! dep_type {
         )?]
         [[$field:ident : $field_ty:ty = $field_val:expr] $($fields:tt)*]
     ) => {
-        $crate::dep_type! {
+        $crate::dep_type_impl_raw! {
             @unroll_fields
             [$([$attr])*] [$vis] [$name] [$obj] [$Id]
             [$($g)*] [$($r)*] [$($w)*]
@@ -926,7 +1224,7 @@ macro_rules! dep_type {
         )?]
         [[$field:ident [$field_ty:ty]] $($fields:tt)*]
     ) => {
-        $crate::dep_type! {
+        $crate::dep_type_impl_raw! {
             @unroll_fields
             [$([$attr])*] [$vis] [$name] [$obj] [$Id]
             [$($g)*] [$($r)*] [$($w)*]
@@ -974,7 +1272,7 @@ macro_rules! dep_type {
         )?]
         [[$event:ident yield $event_args:ty] $($fields:tt)*]
     ) => {
-        $crate::dep_type! {
+        $crate::dep_type_impl_raw! {
             @unroll_fields
             [$([$attr])*] [$vis] [$name] [$obj] [$Id]
             [$($g)*] [$($r)*] [$($w)*]
@@ -1087,6 +1385,7 @@ macro_rules! dep_type {
             impl $($g)* $crate::DepType for $name $($r)* $($w)* {
                 type Id = $Id;
 
+                #[doc(hidden)]
                 fn style__(&mut self) -> &mut $crate::std_option_Option<$crate::Style<$name $($r)*>> {
                     &mut self.core.dep_type_core_style
                 }
@@ -1120,6 +1419,51 @@ macro_rules! dep_type {
 
 #[macro_export]
 macro_rules! dep_obj {
+    (
+        $vis:vis fn $name:ident (self as $this:ident, $arena:ident : $Arena:ty) -> dyn $ty:tt {
+            if mut { $field_mut:expr } else { $field:expr }
+        }
+    ) => {
+        $crate::paste_paste! {
+            fn [< $name _get_obj_ref_priv >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
+                $arena: &'arena_lifetime $Arena,
+                $this: Self
+            ) -> &'arena_lifetime DepObjType {
+                $field.downcast_ref::<DepObjType>().expect("invalid cast")
+            }
+
+            fn [< $name _get_obj_mut_priv >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
+                $arena: &'arena_lifetime mut $Arena,
+                $this: Self
+            ) -> &'arena_lifetime mut DepObjType {
+                $field_mut.downcast_mut::<DepObjType>().expect("invalid cast")
+            }
+
+            #[allow(dead_code)]
+            $vis fn [< $name _ref >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
+                self,
+                $arena: &'arena_lifetime $Arena,
+            ) -> $crate::DepObjRef<'arena_lifetime, DepObjType, $Arena> {
+                $crate::DepObjRef::new($arena, self, Self:: [< $name _get_obj_ref_priv >] )
+            }
+
+            #[allow(dead_code)]
+            $vis fn [< $name _mut >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
+                self,
+                context: &'arena_lifetime mut dyn $crate::dyn_context_Context,
+            ) -> $crate::DepObjMut<'arena_lifetime, DepObjType, $Arena> {
+                $crate::DepObjMut::new(self, context, Self:: [< $name _get_obj_mut_priv >] )
+            }
+
+            #[allow(dead_code)]
+            $vis fn $name <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
+                self,
+                $arena: &'arena_lifetime mut $Arena,
+            ) -> $crate::DepObjRefMut<'arena_lifetime, DepObjType, $Arena> {
+                $crate::DepObjRefMut::new($arena, self, Self:: [< $name _get_obj_mut_priv >] )
+            }
+        }
+    };
     (
         $vis:vis fn $name:ident (self as $this:ident, $arena:ident: $Arena:ty) -> $ty:ty {
             if mut { $field_mut:expr } else { $field:expr }
@@ -1165,51 +1509,6 @@ macro_rules! dep_obj {
             }
         }
     };
-    (
-        $vis:vis dyn fn $name:ident (self as $this:ident, $arena:ident : $Arena:ty) -> $ty:tt {
-            if mut { $field_mut:expr } else { $field:expr }
-        }
-    ) => {
-        $crate::paste_paste! {
-            fn [< $name _get_obj_ref_priv >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
-                $arena: &'arena_lifetime $Arena,
-                $this: Self
-            ) -> &'arena_lifetime DepObjType {
-                $field.downcast_ref::<DepObjType>().expect("invalid cast")
-            }
-
-            fn [< $name _get_obj_mut_priv >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
-                $arena: &'arena_lifetime mut $Arena,
-                $this: Self
-            ) -> &'arena_lifetime mut DepObjType {
-                $field_mut.downcast_mut::<DepObjType>().expect("invalid cast")
-            }
-
-            #[allow(dead_code)]
-            $vis fn [< $name _ref >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
-                self,
-                $arena: &'arena_lifetime $Arena,
-            ) -> $crate::DepObjRef<'arena_lifetime, DepObjType, $Arena> {
-                $crate::DepObjRef::new($arena, self, Self:: [< $name _get_obj_ref_priv >] )
-            }
-
-            #[allow(dead_code)]
-            $vis fn [< $name _mut >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
-                self,
-                context: &'arena_lifetime mut dyn $crate::dyn_context_Context,
-            ) -> $crate::DepObjMut<'arena_lifetime, DepObjType, $Arena> {
-                $crate::DepObjMut::new(self, context, Self:: [< $name _get_obj_mut_priv >] )
-            }
-
-            #[allow(dead_code)]
-            $vis fn $name <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
-                self,
-                $arena: &'arena_lifetime mut $Arena,
-            ) -> $crate::DepObjRefMut<'arena_lifetime, DepObjType, $Arena> {
-                $crate::DepObjRefMut::new($arena, self, Self:: [< $name _get_obj_mut_priv >] )
-            }
-        }
-    };
 }
 
 #[cfg(test)]
@@ -1265,7 +1564,7 @@ mod test {
     }
 
 
-    dep_type! {
+    dep_type_with_builder! {
         #[derive(Debug)]
         struct TestObj1 become obj1 in TestId {
             int_val: i32 = 42,
@@ -1351,7 +1650,11 @@ mod test {
         TestObj1::new(&mut arena, id);
         assert!(id.obj1_ref(&arena).items(TestObj1::COLL).is_empty());
         let change = id.obj1_mut(&mut arena).push(TestObj1::COLL, 7);
-        assert_eq!(change, DepVecChange::Inserted(0 .. 1));
+        let change_match = match &change {
+            DepVecChange::Inserted(x) => *x == (0 .. 1),
+            _ => false,
+        };
+        assert!(change_match, "{:?}", change);
         assert_eq!(id.obj1_ref(&arena).items(TestObj1::COLL)[0], 7);
     }
 }
